/*
 * Based on Processor.java from AndFlmsg for Android by VK2ETA
 *
 * Based on Pskmail from Per Crusefalk and Rein Couperus
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.Tivar;

import java.util.Calendar;
import java.util.concurrent.Semaphore;
import java.io.*;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.Service;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Environment;
import android.os.IBinder;
import android.widget.Toast;


/**
 * @author John Douyere (VK2ETA)
 */
public class Processor extends Service {

    static String application = "TIVAR 1.2"; // Used to preset an empty status
    static String version = "TIVAR 1.2, 2019-07-31";

    static boolean onWindows = true;
    static String HomePath = "";
    static String Dirprefix = "";
    static final String DirTemp = "Temp";
    static final String DirBroadcastLog = "Broadcasts";
    static final String DirImages = "Tivar-Images";
    static final String tempPicfn = "tempicture.jpg";
    static String Separator = "";
    static String Sendline = "";
    static String FileNameString = "";

    //JD temp FIX: init as first modem in list
    static int TxModem = Modem.customModeListInt[0];
    static int RxModem = Modem.customModeListInt[0];

    static int imageTxModemIndex = 0;

    //Semaphores to instruct the Rx Thread to start or stop
    public static Semaphore restartRxModem = new Semaphore(1, false);

    // globals to pass info to gui windows
    static StringBuilder monitor = new StringBuilder();
    static String TermWindow = "";
    static String Status = "";
    static int cpuload;

    // Error handling and logging object
    static loggingclass log;


    public static void processor() {
        //Nothing as this is a service
    }

    @Override
    public void onCreate() {

        // Create error handling class
        log = new loggingclass("Tivar");

        //Initialize Modem (creates the various type of modem objects)
        Modem.ModemInit();

        //Check that we have a current mode, otherwise take the first one in the list (useful when we have a NIL list of custom modes)
        Processor.RxModem = Processor.TxModem = Modem.customModeListInt[Modem.getModeIndex(Processor.RxModem)];

        //Reset frequency and squelch
        Modem.reset();

        //Make sure the display strings are blank
        Processor.monitor = new StringBuilder();
        Processor.TermWindow = "";
    }


    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        // Start the RxThread
        Modem.startmodem();

        //Make sure Android keeps this running even if resources are limited
        //Display the notification in the system bar at the top at the same time
        startForeground(1, Tivar.myNotification);

        return START_NOT_STICKY;

    }


    @Override
    public void onDestroy() {
        // Kill the Rx Modem thread
        Modem.stopRxModem();
    }


    //Post to main terminal window
    public static void PostToTerminal(String text) {
        Processor.TermWindow += text;
        Tivar.mHandler.post(Tivar.addtoterminal);
    }


    //Post to main terminal window
    public static void PostToModem(String text) {
        Processor.monitor.append(text);
        Tivar.mHandler.post(Tivar.addtomodem);
    }


    /**
     * Create or check the necessary folder structure
     */
    public static void handlefolderstructure() {

        //VK2ETA To-Do: add exception here when there is no external storage
        final String defaultPath = Environment.getExternalStorageDirectory().getAbsolutePath();
        // are we on Linux/Android OR Windows?
        if (File.separator.equals("/")) {
            Separator = "/";
            Dirprefix = "/TIVAR.files/";
            onWindows = false;
        } else {
            Separator = "\\";
            Dirprefix = "\\TIVAR.files\\";
            onWindows = true;
        }

        //debug
        //Toast toast = Toast.makeText(Tivar.myContext, "Default: " + HomePath, Toast.LENGTH_LONG);
        //toast.show();

        //Choose the location of the base directory if not defined yet (first run or after de-install)
        HomePath = config.getPreferenceS("BASEPATH", "");
        if (HomePath.length() == 0) {
            //Save new Home Folder
            SharedPreferences.Editor editor = Tivar.mysp.edit();
            editor.putString("BASEPATH", defaultPath);
            // Commit the edits!
            editor.commit();
            //Make folders and copy forms
            makeDirectories(defaultPath);
        } else {
            makeDirectories(HomePath);
        }
    }


    private static void makeDirectories(String newHomePath) {
        try {

            //Then save the new path
            HomePath = newHomePath;

            //Check if base directory exists, create if not
            File baseDir = new File(HomePath + Dirprefix);
            if (!baseDir.isDirectory()) {
                baseDir.mkdir();
            }

            File thisDir;


            //Check if Logs directory exists, create if not
            thisDir = new File(HomePath + Dirprefix + DirBroadcastLog + Separator);
            if (!thisDir.isDirectory()) {
                thisDir.mkdir();
            }

            //Check if Images directory exists, create if not
            thisDir = new File(HomePath + Dirprefix + DirImages + Separator);
            if (!thisDir.isDirectory()) {
                thisDir.mkdir();
            }

            //Check if Temp directory exists, create if not
            thisDir = new File(HomePath + Dirprefix + DirTemp + Separator);
            if (!thisDir.isDirectory()) {
                thisDir.mkdir();
            }
            // Now clears the Temp directory of temporary files
            // This filter only returns files
            FileFilter fileFilter = new FileFilter() {
                public boolean accept(File file) {
                    return file.isFile();
                }
            };
            //Build array of strings containing the file names
            File[] files = thisDir.listFiles(fileFilter);
            for (int i = 0; i < files.length; i++) {
                files[i].delete();
            }

            //Any first time run messages or guides?
            Boolean haveRunOnceBefore = config.getPreferenceB("HAVERUNONCEBEFORE", false);
            if (!haveRunOnceBefore) {
                //
                //Code here for first time run after new install
                //
                //Now save the flag that we have run once
                SharedPreferences.Editor editor = Tivar.mysp.edit();
                editor.putBoolean("HAVERUNONCEBEFORE", true);
                // Commit the edits!
                editor.commit();
            }
        } catch (Exception ex) {
            loggingclass.writelog("Problem when handling Tivar folder structure.", ex, true);
        }
    }


    static double decayaverage(double oldaverage, double newvalue, double factor) {

        double newaverage = oldaverage;
        if (factor > 1) {
            newaverage = (oldaverage * (1 - 1 / factor)) + (newvalue / factor);
        }
        return newaverage;
    }


    //Return current time as a String
    static String myTime() {
        // create a java calendar instance
        Calendar calendar = Calendar.getInstance();

        // get a java.util.Date from the calendar instance.
        // this date will represent the current instant, or "now".
        java.util.Date now = calendar.getTime();

        // a java current time (now) instance
        java.sql.Timestamp currentTimestamp = new java.sql.Timestamp(now.getTime());

        return currentTimestamp.toString().substring(0, 16);
    }

    //Logging
    static void log(String logtext) {
        //           File consolelog = new File (HomePath + Dirprefix + "logfile");
        try {
            // Create file
            FileWriter logstream = new FileWriter(HomePath + Dirprefix + "logfile", true);
            BufferedWriter out = new BufferedWriter(logstream);

            out.write(myTime() + " " + logtext + "\n");
            //Close the output stream
            out.close();

        } catch (Exception e) {//Catch exception if any
            loggingclass.writelog("LogError " + e.getMessage(), null, true);
        }
        Processor.PostToTerminal(myTime() + " " + logtext + "\n");
    }


    @Override
    public IBinder onBind(Intent arg0) {
        // Nothing here, not used
        return null;
    }


}

